"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
/*! *****************************************************************************
Copyright (c) 2016 Tangra Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
***************************************************************************** */
var application = require("application");
var common = require("./push-common");
var utils = require("../utils");
__export(require("./push-common"));
var PUSH_PLATFORM = "gcm";
var MobileServicePush = (function (_super) {
    __extends(MobileServicePush, _super);
    function MobileServicePush() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(MobileServicePush.prototype, "installationId", {
        get: function () {
            return com.microsoft.windowsazure.mobileservices.MobileServiceApplication.getInstallationId(application.android.currentContext);
        },
        enumerable: true,
        configurable: true
    });
    MobileServicePush.prototype.register = function (registrationId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                var futureResult = _this._msPush.register(registrationId);
                utils.futureToPromise(futureResult).then(resolve, reject);
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServicePush.prototype.registerWithTemplate = function (registrationId, templateName, templateBody) {
        return this.registerWithTagsAndTemplate(registrationId, null, templateName, templateBody);
    };
    MobileServicePush.prototype.registerWithTags = function (registrationId, tags) {
        return this.registerWithTagsAndTemplate(registrationId, tags, null, null);
    };
    MobileServicePush.prototype.registerWithTagsAndTemplate = function (registrationId, tags, templateName, templateBody) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                var tagsAsList = null;
                var templates = null;
                if (tags && tags.length > 0) {
                    tagsAsList = java.util.Arrays.asList(tags);
                }
                if (templateName && templateBody) {
                    templates = new java.util.HashMap();
                    templates.put(templateName, new com.microsoft.windowsazure.mobileservices.notifications.InstallationTemplate(templateBody, null));
                }
                var installation = new com.microsoft.windowsazure.mobileservices.notifications.Installation(_this.installationId, PUSH_PLATFORM, registrationId, null, tagsAsList, templates);
                var futureResult = _this._msPush.register(installation);
                utils.futureToPromise(futureResult).then(resolve, reject);
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServicePush.prototype.unregister = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                var futureResult = _this._msPush.unregister();
                utils.futureToPromise(futureResult).then(resolve, reject);
            }
            catch (e) {
                reject(e);
            }
        });
    };
    return MobileServicePush;
}(common.MobileServicePush));
exports.MobileServicePush = MobileServicePush;
