"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
/*! *****************************************************************************
Copyright (c) 2016 Tangra Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
***************************************************************************** */
var common = require("./table-common");
var utils = require("../utils");
var query_1 = require("../query");
__export(require("./table-common"));
var MobileServiceTable = (function (_super) {
    __extends(MobileServiceTable, _super);
    function MobileServiceTable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MobileServiceTable.prototype.read = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._msTable.readWithCompletion(function (queryResult, error) {
                    if (error) {
                        reject(new Error(error.localizedDescription));
                        return;
                    }
                    resolve(utils.getJsObject(queryResult.items));
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServiceTable.prototype.insert = function (item) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._msTable.insertCompletion(utils.getNativeObject(item), function (result, error) {
                    if (error) {
                        reject(new Error(error.localizedDescription));
                        return;
                    }
                    resolve(utils.getJsObject(result));
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServiceTable.prototype.update = function (item) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._msTable.updateCompletion(utils.getNativeObject(item), function (result, error) {
                    if (error) {
                        reject(new Error(error.localizedDescription));
                        return;
                    }
                    resolve(utils.getJsObject(result));
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServiceTable.prototype.deleteById = function (id) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._msTable.deleteWithIdCompletion(id, function (result, error) {
                    if (error) {
                        reject(new Error(error.localizedDescription));
                        return;
                    }
                    resolve();
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServiceTable.prototype.deleteItem = function (item) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._msTable.deleteCompletion(utils.getNativeObject(item), function (result, error) {
                    if (error) {
                        reject(new Error(error.localizedDescription));
                        return;
                    }
                    resolve();
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    MobileServiceTable.prototype.where = function () {
        return new query_1.MobileServiceQuery(this._msTable.query());
    };
    return MobileServiceTable;
}(common.MobileServiceTable));
exports.MobileServiceTable = MobileServiceTable;
