"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*! *****************************************************************************
Copyright (c) 2016 Tangra Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
***************************************************************************** */
var types = require("utils/types");
function futureToPromise(future /* ListenableFuture */) {
    return new Promise(function (resolve, reject) {
        com.google.common.util.concurrent.Futures.addCallback(future, new com.google.common.util.concurrent.FutureCallback({
            onSuccess: function (result) {
                resolve(result);
            },
            onFailure: function (t /* Throwable */) {
                reject(new Error(t.getMessage()));
            }
        }));
    });
}
exports.futureToPromise = futureToPromise;
function getNativeValueForComparison(value) {
    if (types.isNumber(value)) {
        // To my understanding Azure does not use the types strictly
        // So always use the "bigger" type
        if (value.toString().indexOf(".") >= 0) {
            return new java.lang.Double(value);
        }
        else {
            return new java.lang.Long(value);
        }
    }
    else if (value instanceof Date) {
        return new java.util.Date(value.getTime());
    }
    else {
        return value;
    }
}
exports.getNativeValueForComparison = getNativeValueForComparison;
function getNativeObject(jsObject) {
    var parser = new com.google.gson.JsonParser();
    return parser.parse(JSON.stringify(jsObject)).getAsJsonObject();
}
exports.getNativeObject = getNativeObject;
function getJsObject(jsonObject) {
    var gson = new com.google.gson.Gson();
    return JSON.parse(gson.toJson(jsonObject), dateReviver);
}
exports.getJsObject = getJsObject;
function dateReviver(key, value) {
    var isoDateMatcher = /\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+([+-][0-2]\d:[0-5]\d|Z)/;
    if (typeof value === "string" && value.match(isoDateMatcher)) {
        return new Date(value);
    }
    return value;
}
